<?php
/**
 * Bulk Importer Command
 * @since 4.8.0
 * @package modules/NsBulkImporter
**/

namespace Modules\NsBulkImporter\Console\Commands;

use App\Models\OrderPayment;
use App\Models\Product;
use App\Models\ProductCategory;
use App\Services\ProductCategoryService;
use Carbon\Carbon;
use Faker\Factory;
use Illuminate\Console\Command;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Storage;
use PhpCsv\Generator;

class CsvGeneratorcommand extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'nsbulkimporter:generate {type} {--lines=100} {--format=csv}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Will generate sample data for importing.';

    public function handle()
    {
        switch( $this->argument( 'type' ) ) {
            case 'orders': $this->generateOrdersCSV(); break;
        }
    }

    public function generateOrdersCSV()
    {
        $categoryService    =   app()->make( ProductCategoryService::class );
        $orderPayments  =   OrderPayment::get()->map( fn( $payment ) => $payment->identifier )->toArray();
        $categoryChildrens  =   $categoryService->getAllCategoryChildrens();
        $generator  =   new Generator;
        $faker      =   ( new Factory )->create();
        $startRange     =   now()->subMonths(12);
        $dates      =   [];

        while( ! now()->isSameDay( $startRange ) ) {
            $dates[]    =   $startRange->toDateTimeString();
            $startRange->addDay();
        }
                
        $columns    =   [
            'order code'    =>  fn() => 'order-' . $faker->numberBetween(1,10),
            'order shipping' => fn() => $faker->numberBetween(1,5),
            'order discount' => fn() => $faker->numberBetween(0,2),
            'order payment type' => fn() => $faker->randomElement( $orderPayments ),
            'order type' => fn() => $faker->randomElement( ns()->option->get( 'ns_pos_order_types' ) ),
            'order date' => fn() => $faker->randomElement( $dates ),
            'customer name' => fn() => $faker->firstName(),
            'customer email' => fn() => $faker->email(),
            'product name' => fn() => $faker->sentence(2),
            'product price' => fn() => $faker->numberBetween(10,99),
            'product quantity' => fn() => $faker->numberBetween(1,10),
            'product preview' => fn() => $faker->randomElement([
                "https://i5.walmartimages.com/asr/52e47103-c8d9-4cfb-b6d1-2f645f8db854.d1babcce12e6372dd36ca4e2986cbd35.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/38a26b39-c223-411f-a6e2-47090d16556f.fccb8f7045870dd558738059ba9f5f94.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/9ac4579c-d97e-40c2-88f0-f97dd7bbb3c9.cdcc18c9b674d1e273c5036d1c083f38.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/c8a6eebd-c1b5-43a9-a67a-3f444a66b601.30e580b459766635a7a4c5712defcb24.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/ba518fc8-1e1b-4c75-be5c-64b7ef11d623.42b0484fcec68ec243cf78a23f9f7ee8.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/41040f20-fa2a-43b5-85e4-220f43ff8c75.258fc12bd3839fba63af84a80643009b.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/623fb58d-f53d-4f06-93ab-e73c50ccbc71.08f8bbdc87104d97d165528c511ded9b.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/8aa86350-b957-4250-9853-ec1e2ecb9ebc.386fca55611219fa9184e7dff0ef580e.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/805ccff8-50bd-452f-af57-637fe53b36a0.e2a7df4f6fca6dd87d783d5c5029a65a.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/a7a61d2c-aa82-4557-99a1-ec842343a432.df139e77d3acc3dcbec2a0a0dd3e8fee.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/990dffc2-0d23-4237-bdc4-5a530fe29955_1.03f9ea250f8573f80f39bdffa2c15ee0.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/ffc4d099-8e8e-479e-b879-9d55d62c3230.012e252b41ef30b52fa0f9b3fcaa4bb8.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/41040f20-fa2a-43b5-85e4-220f43ff8c75.258fc12bd3839fba63af84a80643009b.jpeg?odnHeight=372&odnWidth=372&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/438d77a1-9fa6-4a1d-abe1-74aad406577a.7b8bf5acf50527976466cfbc94c91185.jpeg?odnHeight=372&odnWidth=372&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/52e47103-c8d9-4cfb-b6d1-2f645f8db854.d1babcce12e6372dd36ca4e2986cbd35.jpeg?odnHeight=372&odnWidth=372&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/4208178c-dda4-40f1-ac94-268f591511d2.5bfb9eaf815aed0f7b7289e8125b2e9f.jpeg?odnHeight=372&odnWidth=372&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/623fb58d-f53d-4f06-93ab-e73c50ccbc71.08f8bbdc87104d97d165528c511ded9b.jpeg?odnHeight=372&odnWidth=372&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/0f559520-3fe3-4720-ae2f-a86e65872acd.19a59cd75bb8f58da466f5211ea7e21a.jpeg?odnHeight=372&odnWidth=372&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/8ae8a4a2-ad04-48c2-ae37-886ba048449e.2b8134a23547b09377c41953e48d2000.jpeg?odnHeight=372&odnWidth=372&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/6abc1e18-db53-4676-b639-eddcf41e5ed5.1fd9eb21e0ef46fcb5ec8f70cdf1cace.jpeg?odnHeight=372&odnWidth=372&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/2c944c17-5370-43e5-938b-261d128ba5ef.4975a45bcddd0a5f17a0b22664626573.jpeg?odnHeight=372&odnWidth=372&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/cc0cbe30-7a23-422f-8720-521b6dd8c7cf.2bf45c6dd1e1beca171f5dfeecb7845a.jpeg?odnHeight=372&odnWidth=372&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/52e47103-c8d9-4cfb-b6d1-2f645f8db854.d1babcce12e6372dd36ca4e2986cbd35.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/438d77a1-9fa6-4a1d-abe1-74aad406577a.7b8bf5acf50527976466cfbc94c91185.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/6abc1e18-db53-4676-b639-eddcf41e5ed5.1fd9eb21e0ef46fcb5ec8f70cdf1cace.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/ed105c0a-1914-4c81-879f-9ecb3782b14c.c74381e7bd11eff8018df72cd53c28a4.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/25282764-b3d5-4137-ac23-b6c945faa913.22aaf1aef2c6eac38bed411f1b7b5cd5.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/d322a434-5bf6-4276-99d6-ac17c14ed68d.24e8b56ea72221ccf0157b46871b8d12.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/e8cfcaf0-61b5-418d-b50f-49601d28718d.60358f5796f36a79cbba9b648c428753.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/ffc4d099-8e8e-479e-b879-9d55d62c3230.012e252b41ef30b52fa0f9b3fcaa4bb8.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/83b9554c-f141-4c86-bfae-b6efe4fe849b.41bba4f6f086914a1ddc1f472706ad28.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/16754ccb-b55a-4b1b-9e5d-c8ae7d0a2a0f.f702ba98f4232ec6318cdef32b555c74.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/7567a6fe-68e3-49bd-a636-f177157744c2.e8f76914cf69dbf38c7c4c26a299b601.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/3afe8c4b-5318-44da-ab1d-1fb5f92c1512.a28c44857efb32445baac00c1868a5c1.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/76c7a826-7300-4500-8d4a-a3e3cb0fdb12_1.db54df82fefdbaf249311b9661502d7a.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/bde65f64-8569-437c-aefe-1bd9abd44a8b.1434afff8d6766e8af3cf193d473dc8c.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/e2cfc4ea-1c7f-4e53-9c2f-39c5677b897e.91c905cbc6d691e9bd5818204bb3ee0c.png?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/0f559520-3fe3-4720-ae2f-a86e65872acd.19a59cd75bb8f58da466f5211ea7e21a.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/816ffd41-b41b-4145-8b0a-9dd08f3ff04d.9d7ef1bb2f3af92fbdec0a805df53de0.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/32f5a49e-ed5a-43a3-9f80-bbd303762bf7.5b2ef868b02ab883a8fe6f58017ff704.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/6666f962-60ab-4b91-899a-2d1b725ba97f_1.d717eed4e553a7e647b5f75a43d01d4b.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/88ce1c97-a5b6-433b-a0ea-b83602982a68.eb54a0a140968298f3dcf9962340499e.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/f488e890-0c69-4295-8459-2f77dd9622b9.bacf354cb0260af9a057ee568edefdce.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/990dffc2-0d23-4237-bdc4-5a530fe29955_1.03f9ea250f8573f80f39bdffa2c15ee0.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/343412c8-a683-4bfe-9892-ccbd169602ff.5add24cddbb9501482f8949cb69e0ac3.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/69b96ffc-5713-4cfe-96e9-e74a6cba6790.f1935cb624dd397363c90580277f8345.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/83b9554c-f141-4c86-bfae-b6efe4fe849b.41bba4f6f086914a1ddc1f472706ad28.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/8d52d6d8-3776-49f5-8f1b-580fe675fc12.3d8d2d27dce4e2f0afbe6f3f940774c2.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/9a311a0b-d4bc-46b6-a2ee-bbd504a26c90.30a59ed3379f1a9a1b22c03fb8d13732.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/9f21fe93-01c5-4f42-b749-37a55ac9e03b.887d990f90f46a1966e284c4238af95c.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/77368af5-019c-4648-b04d-335d7eb74dc3.25646d321e4915b145a320a1549766a4.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF",
                "https://i5.walmartimages.com/asr/1556d9a0-a80c-477e-b58e-641d1905b99c.392f60294168d3c89d4dbf322966b75d.jpeg?odnHeight=580&odnWidth=580&odnBg=FFFFFF"
            ]),
            'product category' => fn() => $faker->randomElement( $categoryChildrens ),
            'product type' => fn() => $faker->randomElement([
                Product::TYPE_DEMATERIALIZED,
                Product::TYPE_MATERIALIZED
            ]),
            'product stock management' => fn() => $faker->randomElement([
                Product::STOCK_MANAGEMENT_DISABLED,
                Product::STOCK_MANAGEMENT_ENABLED,
            ]),
            'product status' => fn() => $faker->randomElement([
                Product::STATUS_AVAILABLE,
                Product::STATUS_UNAVAILABLE
            ])
        ];

        $values     =   [];
        for( $i = 0; $i < $this->option( 'lines' ); $i++ ) {
            $line   =   [];
            foreach( array_keys( $columns ) as $column ) {
                $line[ $column ] = $columns[ $column ]();
            }

            $values[]   =   $line;
        }

        $generator->setArray( $values, array_keys( $columns ) );
        $generator->makeCsv();

        $fileName   =   Str::slug( 'orders-' . date( 'y-m-d' ) ) . '.csv';
        $result     =   Storage::disk( 'public' )->put( $fileName, $generator->getCsv() );

        $this->info( 'The file was exported to : ' . 'public' . DIRECTORY_SEPARATOR . 'storage' . DIRECTORY_SEPARATOR . $fileName );
    }
}
