<?php

namespace Modules\NsBulkImporter\Fields;

use App\Models\Customer;
use App\Models\TaxGroup;
use App\Models\User;
use App\Models\ProductCategory;
use App\Models\CustomerGroup;
use App\Models\PaymentType;
use App\Models\Unit;
use App\Services\FieldsService;
use App\Services\Helper;

class OrderImportFields extends FieldsService
{
    public function get()
    {
        return [
            [
                'label'         =>  __('Separator'),
                'name'          =>  'separator',
                'type'          =>  'text',
                'value'         =>  ',',
                'validation'    =>  'required',
                'description'   =>  __('Define the separator of the file.'),
            ], [
                'label'         =>  __('Default Customer'),
                'name'          =>  'customer_id',
                'type'          =>  'select',
                'validation'    =>  'required',
                'options'       =>  Helper::toJsOptions(Customer::get(), ['id', 'name']),
                'description'   =>  __('In case the customer is missing. Use this default customer.'),
            ], [
                'label'         =>  __('Default Customer Group'),
                'name'          =>  'customer_group_id',
                'type'          =>  'select',
                'validation'    =>  'required',
                'options'       =>  Helper::toJsOptions(CustomerGroup::get(), ['id', 'name']),
                'description'   =>  __('Select the default customer group.'),
            ], [
                'label'         =>  __('Default Cashier'),
                'name'          =>  'cashier_id',
                'type'          =>  'select',
                'validation'    =>  'required',
                'options'       =>  Helper::toJsOptions(User::get(), ['id', 'username']),
                'description'   =>  __('In case the cashier is missing. Use this default cashier.'),
            ], [
                'label'         =>  __('Default Tax Group'),
                'name'          =>  'tax_group_id',
                'type'          =>  'select',
                'validation'    =>  'required',
                'options'       =>  Helper::toJsOptions(TaxGroup::get(), ['id', 'name']),
                'description'   =>  __('In case the tax is missing. Use this default tax.'),
            ], [
                'label'         =>  __('Tax Type'),
                'name'          =>  'tax_type',
                'type'          =>  'select',
                'validation'    =>  'required',
                'options'       =>  Helper::kvToJsOptions([
                    'inclusive' =>  __('Inclusive'),
                    'exclusive' =>  __('Exclusive'),
                ]),
                'description'   =>  __('Define how the taxes are computed.'),
            ],[
                'label'         =>  __('Default Category'),
                'name'          =>  'category_id',
                'type'          =>  'select',
                'validation'    =>  'required',
                'options'       =>  Helper::toJsOptions(ProductCategory::get(), ['id', 'name']),
                'description'   =>  __('Select the default category for products.'),
            ], [
                'label'         =>  __('Default Unit'),
                'name'          =>  'unit_id',
                'type'          =>  'select',
                'validation'    =>  'required',
                'options'       =>  Helper::toJsOptions( Unit::get(), ['id', 'name']),
                'description'   =>  __('In case the unit is missing, this will be used instead.'),
            ], [
                'label'         =>  __('Default Payment Type'),
                'name'          =>  'payment_identifier',
                'type'          =>  'select',
                'validation'    =>  'required',
                'options'       =>  Helper::toJsOptions( PaymentType::get(), ['identifier', 'label']),
                'description'   =>  __('A default payment type should be defined for all orders.'),
            ], 
        ];
    }
}
