<?php

namespace Modules\NsBulkImporter\Http\Controllers;

use App\Http\Controllers\DashboardController;
use App\Services\CurrencyService;
use App\Services\Helper;
use App\Services\ProcurementService;
use Illuminate\Http\Request;
use Modules\NsBulkImporter\Services\ImporterService;

class BulkImporterController extends DashboardController
{
    /**
     * @var ImporterService
     */
    protected $importerService;

    public function __construct(
        ImporterService $importerService
    ) {
        parent::__construct();

        $this->importerService = $importerService;
    }

    public function importer()
    {
        return $this->view('NsBulkImporter::importer', [
            'title'     =>  __('Bulk Importer'),
        ]);
    }

    public function showImportOrders()
    {
        return $this->view('NsBulkImporter::import-orders', [
            'title'     =>  __('Import Orders'),
        ]);
    }

    public function importCustomers()
    {
        return $this->view('NsBulkImporter::import-customers', [
            'title'     =>  __('Import Customers'),
        ]);
    }

    public function supportedAttribute($resource = 'products')
    {
        if ($resource === 'products') {
            return $this->productAttribute();
        } elseif ($resource === 'customers') {
            return $this->customersAttribute();
        } elseif ($resource === 'orders') {
            return $this->ordersAttribute();
        }
    }

    public function productAttribute()
    {
        return Helper::kvToJsOptions([
            'name'          =>  [
                'value'     =>  __('Name'),
                'type'      =>  'text',
            ],
            'category'      =>  [
                'value'     =>  __('Category'),
                'type'      =>  'text',
            ],
            'sku'           =>  [
                'value'     =>  __('SKU'),
                'type'      =>  'text',
            ],
            'barcode'       =>  [
                'value'     =>  __('Barcode'),
                'type'      =>  'text',
            ],
            'unit'          =>  [
                'value'     =>  __('Unit'),
                'type'      =>  'text',
            ],
            'quantity'      =>  [
                'value'     =>  __('Quantity'),
                'type'      =>  'text',
            ],
            'sale_price'    =>  [
                'value'     =>  __('Sale Price'),
                'type'      =>  'text',
            ],
            'purchase_price'    =>  [
                'value'     =>  __('Purchase Price'),
                'type'      =>  'text',
            ],
            'wholesale_price'    =>  [
                'value'     =>  __('WholeSale Price'),
                'type'      =>  'text',
            ],
            'preview'       =>  [
                'value'     =>  __('Preview'),
                'type'      =>  'text',
            ],
            'unit_preview'       =>  [
                'value'     =>  __('Unit Preview'),
                'type'      =>  'text',
            ],
            'category_preview'  =>  [
                'value'     =>  __('Category Preview'),
                'type'      =>  'text',
            ],
            'status'        =>  [
                'value'     =>  __('Status'),
                'type'      =>  'text',
                'options'   =>  [
                    'available',
                    'unavailable',
                ],
            ],
        ]);
    }

    public function customersAttribute()
    {
        return Helper::kvToJsOptions([
            'name'          =>  [
                'value'     =>  __('Name'),
                'type'      =>  'text',
            ],
            'group_name'    =>  [
                'value'     =>  __('Group Name'),
                'type'      =>  'text',
            ],
            'email'           =>  [
                'value'     =>  __('Email'),
                'type'      =>  'text',
            ],
            'surname'          =>  [
                'value'     =>  __('Surname'),
                'type'      =>  'text',
            ],
            'address_1'          =>  [
                'value'     =>  __('Address 1'),
                'type'      =>  'text',
            ],
            'address_2'      =>  [
                'value'     =>  __('Address 2'),
                'type'      =>  'text',
            ],
            'country'    =>  [
                'value'     =>  __('Country'),
                'type'      =>  'text',
            ],
            'phone'    =>  [
                'value'     =>  __('Phone'),
                'type'      =>  'text',
            ],
            'pobox'    =>  [
                'value'     =>  __('PO.box'),
                'type'      =>  'text',
            ],
            'company'       =>  [
                'value'     =>  __('Company'),
                'type'      =>  'text',
            ],
        ]);
    }

    public function ordersAttribute()
    {
        return Helper::kvToJsOptions([
            'name'          =>  [
                'value'     =>  __('Order Name'),
                'type'      =>  'text',
            ],
            'order_code'          =>  [
                'value'     =>  __('Order Reference'),
                'type'      =>  'text',
                'required'  =>  true
            ],
            'order_shipping'          =>  [
                'value'     =>  __('Order Shipping'),
                'type'      =>  'text',
            ],
            'order_discount'          =>  [
                'value'     =>  __('Order Discount'),
                'type'      =>  'text',
            ],
            'order_discount_rate'          =>  [
                'value'     =>  __('Order Discount Rate (%s)'),
                'type'      =>  'text',
            ],
            'order_type'          =>  [
                'value'     =>  __('Order Type'),
                'type'      =>  'text',
            ],
            'order_payment_type'          =>  [
                'value'     =>  __('Order Payment Type'),
                'type'      =>  'text',
            ],
            'order_date'          =>  [
                'value'     =>  __('Order Date'),
                'type'      =>  'text',
            ],
            'customer_name'          =>  [
                'value'     =>  __('Customer Name'),
                'type'      =>  'text',
                'required'  =>  true
            ],
            'customer_email'          =>  [
                'value'     =>  __('Customer Email'),
                'type'      =>  'text',
            ],
            'customer_phone'          =>  [
                'value'     =>  __('Customer Phone'),
                'type'      =>  'text',
            ],
            'product_name'          =>  [
                'value'     =>  __('Product Name'),
                'type'      =>  'text',
                'required'  =>  true,
            ],
            'product_sku'          =>  [
                'value'     =>  __('Product SKU'),
                'type'      =>  'text',
            ],
            'product_barcode'          =>  [
                'value'     =>  __('Product Barcode'),
                'type'      =>  'text',
            ],
            'product_price'          =>  [
                'value'     =>  __('Product Price'),
                'type'      =>  'text',
                'required'  =>  true,
            ],
            'product_quantity'          =>  [
                'value'     =>  __('Product Quantity'),
                'type'      =>  'text',
                'required'  =>  true,
            ],
            'product_status'    =>  [
                'value'     =>  __( 'Product Status' ),
                'type'      =>  'text',
            ],            
            'product_discount'          =>  [
                'value'     =>  __('Product Discount Rate'),
                'type'      =>  'text',
            ],
            'product_type'    =>  [
                'value'     =>  __( 'Product Type' ),
                'type'      =>  'text',
            ],
            'product_stock_management'    =>  [
                'value'     =>  __( 'Product Stock Management' ),
                'type'      =>  'text',
            ],
            'product_barcode_type'    =>  [
                'value'     =>  __( 'Product Barcode Type' ),
                'type'      =>  'text',
            ],
            'product_tax_type'    =>  [
                'value'     =>  __( 'Product Tax Type' ),
                'type'      =>  'text',
            ],
            'product_wholesale_price'          =>  [
                'value'     =>  __('Product Whole Sale Price'),
                'type'      =>  'text',
            ],
            'product_purchase_price'          =>  [
                'value'     =>  __('Product Purchase Price'),
                'type'      =>  'text',
            ],
            'product_preview'          =>  [
                'value'     =>  __('Product Preview (URL)'),
                'type'      =>  'text',
            ],
            'product_unit'          =>  [
                'value'     =>  __('Product Unit'),
                'type'      =>  'text',
            ],
            'product_unit_preview'          =>  [
                'value'     =>  __('Product Unit Preview (URL)'),
                'type'      =>  'text',
            ],
            'product_category'          =>  [
                'value'     =>  __('Product Category'),
                'type'      =>  'text',
            ],
        ]);
    }

    public function importCategories(Request $request)
    {
        return $this->importerService->createCategories(
            $request->input('entries'),
            $request->input('form')
        );
    }

    public function importUnits(Request $request)
    {
        return $this->importerService->createUnits(
            $request->input('entries'),
            $request->input('form')
        );
    }

    public function importProducts(Request $request)
    {
        return $this->importerService->createProducts(
            $request->input('entries'),
            $request->input('form')
        );
    }

    public function procureProducts(Request $request)
    {
        if (! empty($request->input('procurement'))) {
            /**
             * @var ProcurementService
             */
            $procurementService = app()->make(ProcurementService::class);

            /**
             * @var CurrencyService
             */
            $currencyService = app()->make(CurrencyService::class);

            /**
             * retreiving the procurement
             * form before saving that.
             */
            $form = $request->input('procurement');

            /**
             * @todo we need validation here
             */
            $form['products'] = collect($request->input('entries'))->map(function ($product) use ($currencyService) {
                return [
                    'name'                      =>  $product['name'],
                    'product_id'                =>  $product['id'],
                    'gross_purchase_price'      =>  $product['purchase_price']['value'] ?? 0,
                    'purchase_price'            =>  $product['purchase_price']['value'] ?? 0,
                    'net_purchase_price'        =>  $product['purchase_price']['value'] ?? 0,
                    'tax_group_id'              =>  0,
                    'barcode'                   =>  $product['barcode'],
                    'tax_type'                  =>  'inclusive',
                    'tax_value'                 =>  0,
                    'quantity'                  =>  (float) $product['quantity']['value'],
                    'unit_id'                   =>  $product['unit']['id'],
                    'total_purchase_price'      =>  $currencyService->define($product['purchase_price']['value'] ?? 0)
                        ->multiplyBy($product['quantity']['value'])
                        ->getRaw(),
                ];
            });

            return $procurementService->create($form);
        }

        return [
            'status'    =>  'success',
            'message'   =>  __('No procurement is defined. The process has been completed.'),
        ];
    }

    public function importCustomerGroup(Request $request)
    {
        return $this->importerService->createCustomerGroup(
            $request->input('entries'),
            $request->input('form'),
        );
    }

    public function importCustomersFromOrders( Request $request )
    {
        return $this->importerService->createCustomerFromOrder(
            entries: $request->input('entries'),
            form: $request->input('form'),
        );
    }

    public function createCustomer(Request $request)
    {
        return $this->importerService->createCustomer(
            $request->input('entries'),
            $request->input('form'),
        );
    }

    /**
     * Import all the products available on
     * an orders request
     * @param Request $request
     * @return array
     */
    public function importProductsFromOrders( Request $request )
    {
        return $this->importerService->importProductsFromOrders(
            $request->input('entries'),
            $request->input('form'),
        );
    }

    public function importOrders( Request $request )
    {
        return $this->importerService->importOrders(
            $request->input('entries'),
            $request->input('form'),
        );
    }

    public function procureProductsStock( Request $request )
    {
        return $this->importerService->procureProductsFromOrders(
            $request->input('entries'),
            $request->input('form'),
        );
    }
}
