<?php

namespace Modules\NsBulkImporter\Providers;

use App\Classes\Hook;
use App\Services\CustomerService;
use App\Services\ModulesService;
use App\Services\Options;
use App\Services\ProcurementService;
use App\Services\ProductCategoryService;
use App\Services\ProductService;
use App\Services\ProviderService;
use Illuminate\Support\Facades\Event;
use Illuminate\Support\ServiceProvider;
use Modules\NsBulkImporter\Events\NsImporterEvent;
use Modules\NsBulkImporter\Services\ImporterService;
use Modules\NsMultiStore\Events\MultiStoreApiRoutesLoadedEvent;
use Modules\NsMultiStore\Events\MultiStoreWebRoutesLoadedEvent;

class ModuleServiceProvider extends ServiceProvider
{
    public function register()
    {
        Hook::addFilter('ns-dashboard-menus', [NsImporterEvent::class, 'registerMenus'], 30);
        Hook::addFilter('ns.fields', [NsImporterEvent::class, 'registerFields'], 20);

        Event::listen(MultiStoreWebRoutesLoadedEvent::class, function () {
            ModulesService::loadModuleFile('NsBulkImporter', 'Routes/multistore');
        });
        Event::listen(MultiStoreApiRoutesLoadedEvent::class, function () {
            ModulesService::loadModuleFile('NsBulkImporter', 'Routes/api');
        });

        $this->app->singleton(ImporterService::class, function () {
            return new ImporterService(
                app()->make(ProductService::class),
                app()->make(ProductCategoryService::class),
                app()->make(CustomerService::class),
                app()->make(ProviderService::class),
                app()->make(Options::class),
                app()->make(ProcurementService::class),
            );
        });
    }
}
