<div id="dashboard-header" class="w-full flex flex-col-reverse md:flex-row justify-between p-4">
    <div class="flex items-center my-2 md:my-0">
        <div>
            <div @click="toggleSideMenu()" class="hover:shadow-lg hover:border-opacity-0 border ns-inset-button rounded-full multi-h-10 multi-w-10 cursor-pointer font-bold text-2xl justify-center items-center flex">
                <i class="las la-bars"></i>
            </div>
        </div>
        <div class="multi-ml-3">
            @if( ns()->store->isMultiStore() )
            <h2 class="font-bold text-2xl primary">{{ ns()->store->getCurrentStore()->name }}</h2>
            @else
            <h2 class="font-bold text-2xl primary">{{ __m( 'Multi Stores', 'NsMultiStore' ) }}</h2>
            @endif
        </div>
    </div>
    <div class="top-tools-side justify-between md:justify-end flex w-full md:w-auto items-center md:-mx-2">
        <div clss="multi-px-2">
            <ns-notifications></ns-notifications>
        </div>
        @if ( ! ns()->store->subDomainsEnabled() || ! ns()->store->isMultiStore() )
        <div class="multi-px-2">
            <a href="{{ route( 'ns.multistore-stores' ) }}" class="hover:primary hover:shadow-lg hover:border-opacity-0 rounded-full multi-h-12 multi-w-12 cursor-pointer font-bold text-2xl justify-center items-center flex border ns-inset-button">
                <i class="las la-store"></i>
            </a>
        </div>
        <div class="multi-px-2">
            <ns-multistore-selector inline-template class="rounded-lg flex border multi-py-2 justify-center hover:border-opacity-0 cursor-pointer hover:shadow-lg ns-inset-button">
                <div @click="toggleStoreSelector()" class="flex justify-between items-center flex-shrink-0">
                    <span class="hidden md:inline-block px-2">{{ __m( 'Stores', 'NsMultiStore' ) }}</span>
                    <div class="px-2">
                        <div class="multi-w-8 multi-h-8 rounded-full bg-info-tertiary text-white flex items-center justify-center">
                            {{ Auth::user()->roles->map( fn( $role ) => $role->total_stores )->sum() }}
                        </div>
                    </div>
                </div>
            </ns-multistore-selector>
        </div>
        @endif
        <div class="px-2">
            <div @click="toggleMenu()" :class="menuToggled ? 'toggled border-transparent shadow-lg rounded-t-lg' : 'untoggled rounded-lg'" class="ns-avatar w-32 md:w-56 flex flex-col border py-2 justify-center hover:border-opacity-0 cursor-pointer hover:shadow-lg">
                <ns-avatar 
                    display-name="{{ Auth::user()->username }}"
                    url="{{ Auth::user()->attribute ? Auth::user()->attribute->avatar_link : asset( 'images/user.png' ) }}"></ns-avatar>
            </div>
            <div v-cloak class="w-32 md:w-56 shadow-lg flex z-10 absolute -mb-2 rounded-br-lg rounded-bl-lg overflow-hidden" v-if="menuToggled">
                <ul class="primary w-full ns-vertical-menu">
                    @if ( Auth::user()->allowedTo([ 'manage.profile' ]) )
                    <li class="hover:text-primary"><a class="block px-2 py-1" href="{{ ns()->route( 'ns.dashboard.users.profile' ) }}"><i class="las text-lg mr-2 la-user-tie"></i> {{ __m( 'Profile', 'NsMultiStore' ) }}</a></li>
                    @endif
                    <li class="hover:text-primary"><a class="block px-2 py-1" href="{{ ns()->route( 'ns.logout' ) }}"><i class="las la-sign-out-alt mr-2"></i> {{ __m( 'Logout', 'NsMultiStore' ) }}</a></li>
                </ul>
            </div>
        </div>
    </div>
</div>

@section( 'layout.dashboard.footer.inject' )
    @parent
    <script type="text/x-template" id="store-selection-template">
        <div class="ns-box shadow-xl w-95vw md:w-2/3-screen lg:w-2/5-screen xl:w-1/3-screen h-95vh md:h-2/3-screen lg:h-2/3-screen overflow-hidden flex flex-col">
            <div class="header border-b p-2 flex justify-between items-center ns-box-header">
                <h3 class="font-semibold primary">{{ __m( 'Stores', 'NsMultiStore' ) }}</h3>
                <div>
                    <ns-close-button @click="close()"></ns-close-button>
                </div>
            </div>
            <div class="overflow-y-auto flex-auto ns-box-body">
                <div v-if="hasLoaded === 0" class="h-full w-full flex items-center justify-center">
                    <ns-spinner></ns-spinner>
                </div>
                <div v-if="hasLoaded && stores.length === 0" class="h-full w-full flex-col flex items-center justify-center">
                    <p>{{ __m( 'No store has been created.', 'NsMultiStore' ) }}</p><p>{{ __m( 'Or you don\'t have a valid access to any store.', 'NsMultiStore' ) }}</p>
                </div>
                <div v-if="hasLoaded && stores.length > 0" class="grid grid-cols-3 w-full">
                    @if ( ns()->store->subDomainsEnabled() )
                    <a :href="'{{ ns()->store->baseProtocol() }}' + store.slug + '.{{ ns()->store->baseDomainName() }}'" v-for="store of stores" class="border bg-blue-800 cursor-pointer border-gray-200 h-40 relative">
                        <div class="h-full w-full object-contain overflow-hidden flex items-center justify-center">
                            <img v-if="store.thumb" :src="store.thumb" :alt="store.name">
                            <img v-if="! store.thumb" class="w-24" :src="'/modules/nsmultistore/assets/images/shop.png'" :alt="store.name">
                        </div>
                        <div class="h-16 bottom-0 absolute w-full z-10 p-2 flex items-center flex-col justify-center store-name font-semibold" style="background: rgb(0,0,0);
background: linear-gradient(0deg, rgba(0,0,0,0.8379726890756303) 0%, rgba(0,0,0,0.7147233893557423) 32%, rgba(0,212,255,0) 100%);">
                            <span>@{{ store.name }}</span>
                        </div>
                    </a>
                    @else
                    <a :href="'{{ url( '/dashboard/store' ) }}/' + store.id" v-for="store of stores" class="border bg-blue-800 cursor-pointer border-gray-200 h-40 relative">
                        <div class="h-full w-full object-contain overflow-hidden flex items-center justify-center">
                            <img v-if="store.thumb" :src="store.thumb" :alt="store.name">
                            <img v-if="! store.thumb" class="w-24" :src="'/modules/nsmultistore/assets/images/shop.png'" :alt="store.name">
                        </div>
                        <div class="h-16 bottom-0 absolute w-full z-10 p-2 flex items-center flex-col justify-center store-name font-semibold" style="background: rgb(0,0,0);
background: linear-gradient(0deg, rgba(0,0,0,0.8379726890756303) 0%, rgba(0,0,0,0.7147233893557423) 32%, rgba(0,212,255,0) 100%);">
                            <span>@{{ store.name }}</span>
                        </div>
                    </a>
                    @endif
                </div>
            </div>
            <div class="flex w-full ns-box-footer border-t">
                <div class="flex-auto ns-button hover-success">
                    <a href="{{ route( 'ns.multistore-stores' ) }}" class="cursor-pointer flex-auto text-2xl h-16 p-2 flex items-center justify-center">
                        <i class="las la-store mr-2"></i>
                        <span>{{ __m( 'Stores', 'NsMultiStore' ) }}</span>
                    </a>
                </div>
                <div class="ns-button hover-warning flex-auto">
                    <a class="cursor-pointer text-2xl h-16 p-2 flex items-center justify-center" @click="close()" >
                        <i class="las la-times mr-2"></i>
                        <span>{{ __m( 'Cancel', 'NsMultiStore' ) }}</span>
                    </a>
                </div>
            </div>
        </div>
    </script>
    <script src="{{ asset( 'modules/nsmultistore/js/header.js' ) }}"></script>
@endsection