<?php

namespace Modules\NsMultiStore\Tests\Feature;

use App\Classes\Hook;
use App\Models\Option;
use App\Models\OrderPayment;
use App\Models\OrderProductRefund;
use App\Models\Product;
use App\Models\Role;
use App\Services\CurrencyService;
use Illuminate\Foundation\Testing\WithFaker;
use Laravel\Sanctum\Sanctum;
use Modules\NsMultiStore\Models\Store;
use Tests\TestCase;

class StoreCreateTest extends TestCase
{
    use WithFaker;

    /**
     * A basic feature test example.
     *
     * @return void
     */
    public function testCreateStore()
    {
        Sanctum::actingAs(
            Role::namespace('admin')->users->first(),
            ['*']
        );

        $response = $this->withSession($this->app['session']->all())
            ->json('POST', 'api/nexopos/v4/crud/ns.multistore', [
                'name'  =>  $this->faker->company(),
                'general'  => [
                    'status' => Store::STATUS_OPENED,
                    'roles_id' => Role::get()->map( fn( $role ) => $role->id )->toArray(),
                ]
            ]);

        $response->assertJson([
            'status'    =>  'success',
        ]);
    }

    /**
     * A basic feature test example.
     *
     * @return void
     */
    public function testCreateStoresWithSameName()
    {
        Sanctum::actingAs(
            Role::namespace('admin')->users->first(),
            ['*']
        );

        $company    =   $this->faker->company();

        $response = $this->withSession($this->app['session']->all())
            ->json('POST', 'api/nexopos/v4/crud/ns.multistore', [
                'name'  =>  $company,
                'general'  => [
                    'status' => Store::STATUS_OPENED,
                    'roles_id' => Role::get()->map( fn( $role ) => $role->id )->toArray(),
                ]
            ]);

        $response->assertJson([
            'status'    =>  'success',
        ]);

        $response = $this->withSession($this->app['session']->all())
            ->json('POST', 'api/nexopos/v4/crud/ns.multistore', [
                'name'  =>  $company,
                'general'  => [
                    'status' => Store::STATUS_OPENED,
                    'roles_id' => Role::get()->map( fn( $role ) => $role->id )->toArray(),
                ]
            ]);

        $response->assertJson([
            'status'    =>  'failed',
        ]);
    }

    /**
     * A basic feature test example.
     *
     * @return void
     */
    public function testDeleteStore()
    {
        Sanctum::actingAs(
            Role::namespace('admin')->users->first(),
            ['*']
        );

        $company    =   $this->faker->company();

        $response = $this->withSession($this->app['session']->all())
            ->json('POST', 'api/nexopos/v4/crud/ns.multistore', [
                'name'  =>  $company,
                'general'  => [
                    'status' => Store::STATUS_OPENED,
                    'roles_id' => Role::get()->map( fn( $role ) => $role->id )->toArray(),
                ]
            ]);

        $response->assertJson([
            'status'    =>  'success',
        ]);

        $store  =   json_decode( $response->getContent(), true );

        ns()->store->setStore( Store::find( $store[ 'entry' ][ 'id' ] ) );

        /**
         * Will check if by switching the store
         * we're now pointing to the new store options
         */
        $optionTableName    =   'store_' . $store[ 'entry' ][ 'id' ] . '_' . 'nexopos_options';

        Hook::addFilter( 'ns-table-name', [ ns()->store, 'prefixTable' ]);
        Hook::addFilter( 'ns-model-table', [ ns()->store, 'prefixTable' ]);

        $this->assertTrue(
            ( new Option )->getTable() === $optionTableName,
            'The Option instance is not pointing to the store options'
        );

        Hook::removeAllFilters( 'ns-table-name' );
        Hook::removeAllFilters( 'ns-model-table' );

        $response = $this->withSession($this->app['session']->all())
            ->json('DELETE', 'api/nexopos/v4/crud/ns.multistore/' . $store[ 'entry' ][ 'id' ] );

        $response->assertJson([
            'status'    =>  'success',
        ]);
    }
}
