<?php
namespace Modules\NsStockTransfers\Crud;

use Illuminate\Support\Facades\Auth;
use Illuminate\Http\Request;
use App\Services\CrudService;
use App\Services\Users;
use App\Exceptions\NotAllowedException;
use App\Models\User;
use App\Services\CrudEntry;
use App\Services\Helper;
use TorMorten\Eventy\Facades\Events as Hook;
use Exception;
use Modules\NsMultiStore\Models\Store;
use Modules\NsStockTransfers\Models\StockTransfer;
use Modules\NsStockTransfers\Models\StockTransferProduct;

class StockRequestCrud extends StockTransferCrud
{
    /**
     * define the base table
     * @param  string
     */
    protected $table      =   'nexopos_stock_transfers';

    /**
     * default slug
     * @param  string
     */
    protected $slug   =   'stock-requests';

    /**
     * Define namespace
     * @param  string
     */
    protected $namespace  =   'ns.stocktransfers-requests';

    /**
     * Model Used
     * @param  string
     */
    protected $model      =   StockTransfer::class;

    /**
     * Define permissions
     * @param  array
     */
    protected $permissions  =   [
        'create'    =>  true,
        'read'      =>  true,
        'update'    =>  true,
        'delete'    =>  true,
    ];

    /**
     * Adding relation
     * @param  array
     */
    public $relations   =  [
            ];

    /**
     * all tabs mentionned on the tabs relations
     * are ignored on the parent model.
     */
    protected $tabsRelations    =   [
        // 'tab_name'      =>      [ YourRelatedModel::class, 'localkey_on_relatedmodel', 'foreignkey_on_crud_model' ],
    ];

    /**
     * Pick
     * Restrict columns you retreive from relation.
     * Should be an array of associative keys, where 
     * keys are either the related table or alias name.
     * Example : [
     *      'user'  =>  [ 'username' ], // here the relation on the table nexopos_users is using "user" as an alias
     * ]
     */
    public $pick        =   [];

    /**
     * Define where statement
     * @var  array
    **/
    protected $listWhere    =   [];

    /**
     * Define where in statement
     * @var  array
     */
    protected $whereIn      =   [];

    /**
     * Fields which will be filled during post/put
     */
        public $fillable    =   [];

    /**
     * Define Constructor
     * @param  
     */
    public function __construct()
    {
        parent::__construct();

        Hook::addFilter( $this->namespace . '-crud-actions', [ $this, 'setActions' ], 10, 2 );
    }

    /**
     * Fields
     * @param  object/null
     * @return  array of field
     */
    public function getForm( $entry = null ) 
    {
        $sourceIsEditing    =   $entry !== null && ( int ) $entry->from_store === ( int ) Store::current()->id && $entry->type === StockTransfer::TYPE_REQUEST;

        $stores      =   Store::status( Store::STATUS_OPENED )
            ->where( 'id', $sourceIsEditing ? '=' : '<>', ns()->store->current()->id )
            ->get()
            ->map( function( $store ) {
                return [
                    'label' =>  $store->name,
                    'value' =>  $store->id
                ];
            });

        $status     =   [
            'draft'     =>  __( 'Draft' ),
            'pending'   =>  __( 'Pending' ),
        ];

        if ( $entry !== null && $entry->status === 'ongoing' ) {
            $status[ 'ongoing' ]    =   __( 'Ongoing' );
        }

        $status     =   Helper::kvToJsOptions( $status );

        return [
            'main' =>  [
                'label'         =>  __( 'Name' ),
                'name'          =>  'name',
                'value'         =>  $entry->name ?? '',
                'description'   =>  __( 'Provide a name to the resource.' )
            ],
            'products'          =>  $entry !== null ? $entry->products->map( function( StockTransferProduct $stockTransferProduct ) use ( $entry ) {
                /**
                 * We would like fetch product
                 * from the destination store. As the current
                 * store is requesting from a remote store.
                 */
                ns()->store->setStore( Store::find( $entry->from_store ) );

                $stockTransferProduct->unit_quantity     =   $this->productService->getUnitQuantity( 
                    $stockTransferProduct->product_id, $stockTransferProduct->unit_id
                );

                $stockTransferProduct->unit_quantities   =   $this->productService->getProductUnitQuantities( $stockTransferProduct->product );

                return $stockTransferProduct;
            }) : [],
            'tabs'  =>  [
                'general'   =>  [
                    'label'     =>  __( 'General' ),
                    'fields'    =>  [
                        [
                            'type'          =>  'select',
                            'name'          =>  'from_store',
                            'label'         =>  __( 'From Store' ),
                            'description'   =>  __( 'Select the store from where the stock should come from.' ),
                            'validation'    =>  'required',
                            'disabled'      =>  $sourceIsEditing,
                            'value'         =>  $entry->from_store ?? '',
                            'options'       =>  $stores,
                        ], [
                            'type'          =>  'select',
                            'name'          =>  'status',
                            'disabled'      =>  $sourceIsEditing,
                            'label'         =>  __( 'Request Status' ),
                            'description'   =>  __( 'New requests starts with a pending status.' ),
                            'validation'    =>  'required',
                            'value'         =>  $entry->status ?? 'draft',
                            'options'       =>  $status,
                        ], [
                            'type'          =>  'hidden',
                            'name'          =>  'type',
                            'label'         =>  __( 'Request Type' ),
                            'description'   =>  __( 'Select wether the store is requesting or transfering stock.' ),
                            'validation'    =>  'required',
                            'value'         =>   'request',
                        ], [
                            'type'          =>  'hidden',
                            'name'          =>  'to_store',
                            'label'         =>  __( 'To Store' ),
                            'description'   =>  __( 'Select the source store where the stock transfer goes.' ),
                            'validation'    =>  'required',
                            'value'         =>  $entry instanceof StockTransfer ? $entry->to_store : ns()->store->current()->id
                        ], 
                    ]
                ],
                'table'     =>  [
                    'label'     =>  __( 'Products' ),
                    'fields'    =>  [],
                ],
            ]
        ];
    }

    /**
     * Return the label used for the crud 
     * instance
     * @return  array
    **/
    public function getLabels()
    {
        return [
            'list_title'            =>  __( 'Stock Transfers List' ),
            'list_description'      =>  __( 'Display all stock transfers.' ),
            'no_entry'              =>  __( 'No stock transfers has been registered' ),
            'create_new'            =>  __( 'Add a new stock transfer' ),
            'create_title'          =>  __( 'Create a new stock transfer' ),
            'create_description'    =>  __( 'Register a new stock transfer and save it.' ),
            'edit_title'            =>  __( 'Edit stock transfer' ),
            'edit_description'      =>  __( 'Modify  Stock Transfer.' ),
            'back_to_list'          =>  __( 'Return to Stock Transfers' ),
        ];
    }

    /**
     * Check whether a feature is enabled
     * @return  boolean
    **/
    public function isEnabled( $feature ): bool
    {
        return false; // by default
    }

    public function hook( $query ): void
    {
        $currentStore   =   ns()->store->current()->id;
        
        $query->orderBy( 'updated_at', 'desc' );
        $query
            ->where( 'from_store', $currentStore )
            ->where( 'to_store', $currentStore );
    }

    /**
     * We don't want the table name to be hookable
     * as it will run on the warehouse level
     * @return string
     */
    protected function hookTableName( $tableName ): string
    {
        return $tableName;
    }

    /**
     * Filter POST input fields
     * @param  array of fields
     * @return  array of fields
     */
    public function filterPostInputs( $inputs )
    {
        return $inputs;
    }

    /**
     * Filter PUT input fields
     * @param  array of fields
     * @return  array of fields
     */
    public function filterPutInputs( $inputs, StockTransfer $entry )
    {
        return $inputs;
    }

    /**
     * Before saving a record
     * @param  Request $request
     * @return  void
     */
    public function beforePost( $request )
    {
        if ( $this->permissions[ 'create' ] !== false ) {
            ns()->restrict( $this->permissions[ 'create' ] );
        } else {
            throw new NotAllowedException;
        }

        return $request;
    }

    /**
     * After saving a record
     * @param  Request $request
     * @param  StockTransfer $entry
     * @return  void
     */
    public function afterPost( $request, StockTransfer $entry )
    {
        return $request;
    }

    
    /**
     * get
     * @param  string
     * @return  mixed
     */
    public function get( $param )
    {
        switch( $param ) {
            case 'model' : return $this->model ; break;
        }
    }

    /**
     * Before updating a record
     * @param  Request $request
     * @param  object entry
     * @return  void
     */
    public function beforePut( $request, $entry )
    {
        if ( $this->permissions[ 'update' ] !== false ) {
            ns()->restrict( $this->permissions[ 'update' ] );
        } else {
            throw new NotAllowedException;
        }

        return $request;
    }

    /**
     * After updating a record
     * @param  Request $request
     * @param  object entry
     * @return  void
     */
    public function afterPut( $request, $entry )
    {
        return $request;
    }

    /**
     * Before Delete
     * @return  void
     */
    public function beforeDelete( $namespace, $id, $model ) {
        if ( $namespace == 'ns.stocktransfers' ) {
            /**
             *  Perform an action before deleting an entry
             *  In case something wrong, this response can be returned
             *
             *  return response([
             *      'status'    =>  'danger',
             *      'message'   =>  __( 'You\re not allowed to do that.' )
             *  ], 403 );
            **/
            if ( $this->permissions[ 'delete' ] !== false ) {
                ns()->restrict( $this->permissions[ 'delete' ] );
            } else {
                throw new NotAllowedException;
            }
        }
    }

    /**
     * Define Columns
     * @return  array of columns configuration
     */
    public function getColumns() {
        return [
            'name'  =>  [
                'label'  =>  __( 'Name' ),
                '$direction'    =>  '',
                '$sort'         =>  false
            ],
            'status'  =>  [
                'label'  =>  __( 'Status' ),
                '$direction'    =>  '',
                '$sort'         =>  false
            ],
            'created_at'  =>  [
                'label'  =>  __( 'Created At' ),
                '$direction'    =>  '',
                '$sort'         =>  false
            ],
            'user_username'  =>  [
                'label'  =>  __( 'Author' ),
                '$direction'    =>  '',
                '$sort'         =>  false
            ]
        ];
    }

    /**
     * Define actions
     */
    public function setActions( CrudEntry $entry )
    {
        // Don't overwrite
        $entry->{ '$checked' }  =   false;
        $entry->{ '$toggled' }  =   false;
        $entry->{ '$id' }       =   $entry->id;    

        // you can make changes here
        $entry->addAction( 'edit', [
            'label'         =>      __( 'Edit' ),
            'namespace'     =>      'edit',
            'type'          =>      'GOTO',
            'url'           =>      url( '/dashboard/' . '' . '/edit/' . $entry->id )
        ]);

        $entry->addAction( 'delete', [
            'label'     =>  __( 'Delete' ),
            'namespace' =>  'delete',
            'type'      =>  'DELETE',
            'url'       =>  url( '/api/nexopos/v4/crud/ns.stocktransfers/' . $entry->id ),
            'confirm'   =>  [
                'message'  =>  __( 'Would you like to delete this ?' ),
            ]
        ]);

        return $entry;
    }

    
    /**
     * Bulk Delete Action
     * @param    object Request with object
     * @return    false/array
     */
    public function bulkAction( Request $request ) 
    {
        /**
         * Deleting licence is only allowed for admin
         * and supervisor.
         */

        if ( $request->input( 'action' ) == 'delete_selected' ) {

            /**
             * Will control if the user has the permissoin to do that.
             */
            if ( $this->permissions[ 'delete' ] !== false ) {
                ns()->restrict( $this->permissions[ 'delete' ] );
            } else {
                throw new NotAllowedException;
            }

            $status     =   [
                'success'   =>  0,
                'failed'    =>  0
            ];

            foreach ( $request->input( 'entries' ) as $id ) {
                $entity     =   $this->model::find( $id );
                if ( $entity instanceof StockTransfer ) {
                    $entity->delete();
                    $status[ 'success' ]++;
                } else {
                    $status[ 'failed' ]++;
                }
            }
            return $status;
        }

        return Hook::filter( $this->namespace . '-catch-action', false, $request );
    }

    /**
     * get Links
     * @return  array of links
     */
    public function getLinks(): array
    {
        return  [
            'list'      =>  ns()->url( 'dashboard/' . 'stock-transfers' ),
            'create'    =>  ns()->url( 'dashboard/' . 'stock-transfers/create/request' ),
            'edit'      =>  ns()->url( 'dashboard/' . 'stock-transfers/edit/request/' ),
            'post'      =>  ns()->url( 'api/nexopos/v4/crud/' . 'ns.stocktransfers-request' ),
            'put'       =>  ns()->url( 'api/nexopos/v4/crud/' . 'ns.stocktransfers-request/{id}' . '' ),
        ];
    }

    /**
     * Get Bulk actions
     * @return  array of actions
    **/
    public function getBulkActions(): array
    {
        return Hook::filter( $this->namespace . '-bulk', [
            [
                'label'         =>  __( 'Delete Selected Groups' ),
                'identifier'    =>  'delete_selected',
                'url'           =>  ns()->route( 'ns.api.crud-bulk-actions', [
                    'namespace' =>  $this->namespace
                ])
            ]
        ]);
    }

    /**
     * get exports
     * @return  array of export formats
    **/
    public function getExports()
    {
        return [];
    }
}