<?php
namespace Modules\NsStockTransfers\Events;

use Modules\NsStockTransfers\Crud\StockRequestCrud;
use Modules\NsStockTransfers\Crud\StockTransferCrud;
use Modules\NsStockTransfers\Jobs\HandleStockTransferJob;
use Modules\NsStockTransfers\Models\StockTransfer;
use Modules\NsStockTransfers\Services\StockTransferService;

/**
 * Register Events
**/
class NsStockTransfersEvent
{
    public static function dashboardMenus( $menus ) 
    {
        if ( ns()->store->isMultiStore() ) {
            return array_insert_after( $menus, 'inventory', [
                'ns.transfers'      =>      [
                    'label'     =>  __( 'Stock Transfer' ),
                    'icon'      =>  'la-exchange-alt',
                    'childrens' =>  [
                        'ns.transfers.list' =>  [
                            'label'         =>  __( 'All Transfers' ),
                            'permission'    =>  'stocktransfers.read.transaction',
                            'href'          =>  ns()->route( 'ns.stocktransfer-list' ),
                        ],
                        'ns.transfers.transfers' =>  [
                            'label'         =>  __( 'New Transfer' ),
                            'permission'    =>  'stocktransfers.create.transfer',
                            'href'          =>  ns()->route( 'ns.stocktransfer-transfers' ),
                        ],
                        'ns.transfers.requests' =>  [
                            'label'         =>  __( 'New Request' ),
                            'permission'    =>  'stocktransfers.create.request',
                            'href'          =>  ns()->route( 'ns.stocktransfer-requests' ),
                        ],
                    ]
                ]
            ]);
        } else {
            $menus  =   array_insert_after( $menus, 'ns.multistore-stores', [
                'ns.stocktransfers-settings'     =>  [
                    'label'     =>  __m( 'Transfers Settings', 'NsStockTransfers' ),
                    'href'      =>  route( 'ns.stocktransfers-settings' ),
                    'icon'      =>  'la-tools'
                ]
            ]);

            return $menus;
        }

        return $menus;
    }

    /**
     * @deprecated
     */
    public static function handleTransactionBeforeApproval( TransactionBeforeApproveEvent $event )
    {
        /**
         * this means the source store has approved the request
         * and therefore we should deplete the stock
         */
        if ( $event->transaction->type === StockTransfer::TYPE_REQUEST ) {
            /**
             * @var StockTransferService
             */
            $service        =       app()->make( StockTransferService::class );
            $service->processTransaction( $event->transaction ); // will deplete if transaction is an ongoing transfer
        }
    }

    public static function registerCrud( $crud )
    {
        switch( $crud ) {
            case 'ns.stocktransfers-transfers': return StockTransferCrud::class;
            case 'ns.stocktransfers-requests': return StockRequestCrud::class;
            default: return $crud;
        }
    }

    public static function emitNotifications( $event ) 
    {
        /**
         * @var StockTransferService
         */
        $service        =       app()->make( StockTransferService::class );
        $service->notify( $event->transaction );
    }
}