<?php
/**
 * Table Migration
 * @package  4.0
**/

namespace Modules\NsStockTransfers\Migrations;

use App\Models\Permission;
use App\Models\Role;
use Illuminate\Support\Facades\Schema;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Database\Migrations\Migration;

class CreateNewPermissionsDec27 extends Migration
{
    public $multistore      =   false;

    /**
     * Run the migrations.
     *
     * @return  void
     */
    public function up()
    {
        $createStockTransfers           =   Permission::withNamespace( 'stocktransfers.create.transfer' )->firstOr( function() {
            $permission                 =   new Permission;
            $permission->namespace      =    'stocktransfers.create.transfer';
            $permission->name           =    __( 'Create Stock Transfer' );
            $permission->description    =    __( 'Allow the user to create stock transfers.' );
            return $permission->save();
        });

        $createStockRequests        =   Permission::withNamespace( 'stocktransfers.create.request' )->firstOr( function() {
            $permission                 =   new Permission;
            $permission->namespace      =    'stocktransfers.create.request';
            $permission->name           =    __( 'Create Stock Request' );
            $permission->description    =    __( 'Allow the user to create stock requests.' );
            return $permission->save();
        });

        Role::namespace( 'admin' )->addPermissions([
            $createStockTransfers,
            $createStockRequests
        ]);

        Role::namespace( 'nexopos.store.administrator' )->addPermissions([
            $createStockTransfers,
            $createStockRequests
        ]);
    }

    /**
     * Reverse the migrations.
     *
     * @return  void
     */
    public function down()
    {
        Permission::whereIn( 'namespace', [
            'stocktransfers.create.transfer',
            'stocktransfers.create.request',
        ])->each( function( $permission ) {
            $permission->removeFromRoles();
            $permission->delete();
        });
    }
}
