<?php
/**
 * Table Migration
 * @package  4.0
**/

namespace Modules\NsStockTransfers\Migrations;

use App\Models\Permission;
use App\Models\Role;
use Illuminate\Support\Facades\Schema;
use Illuminate\Database\Schema\Blueprint;
use Illuminate\Database\Migrations\Migration;

class CreatePermissions extends Migration
{
    public $multistore      =   false;
    /**
     * Run the migrations.
     *
     * @return  void
     */
    public function up()
    {
        $createTransactionPermission        =   Permission::withNamespace( 'stocktransfers.create.transaction' )->firstOr( function() {
            $permission                 =   new Permission;
            $permission->namespace      =   'stocktransfers.create.transaction';
            $permission->name           =   __( 'Create Stock Transaction' );
            $permission->description    =   __( 'Allow a role to create a transaction (either transfer or request).' );
            return $permission->save();
        });

        $readTransactionPermission          =   Permission::withNamespace( 'stocktransfers.read.transaction' )->firstOr( function() {
            $permission                =   new Permission;
            $permission->namespace      =   'stocktransfers.read.transaction';
            $permission->name           =   __( 'Read Stock Transaction' );
            $permission->description    =   __( 'Allow a role to check a transaction (either transfer or request).' );
            return $permission->save();
        });

        $updateTransactionPermission        =   Permission::withNamespace( 'stocktransfers.update.transaction' )->firstOr( function() {
            $permission                =   new Permission;
            $permission->namespace      =   'stocktransfers.update.transaction';
            $permission->name           =   __( 'Update Stock Transaction' );
            $permission->description    =   __( 'Allow a role to modify a transaction (either transfer or request).' );
            return $permission->save();
        });

        $deleteTransactionPermission        =   Permission::withNamespace( 'stocktransfers.delete.transaction' )->firstOr( function() {
            $permission                =   new Permission;
            $permission->namespace      =   'stocktransfers.delete.transaction';
            $permission->name           =   __( 'Delete Stock Transaction' );
            $permission->description    =   __( 'Allow a role to delete a transaction (either transfer or request).' );
            return $permission->save();
        });

        Role::namespace( 'admin' )->addPermissions([
            $createTransactionPermission,
            $readTransactionPermission,
            $updateTransactionPermission,
            $deleteTransactionPermission,
        ]);

        Role::namespace( 'nexopos.store.administrator' )->addPermissions([
            $createTransactionPermission,
            $readTransactionPermission,
            $updateTransactionPermission,
            $deleteTransactionPermission,
        ]);
    }

    /**
     * Reverse the migrations.
     *
     * @return  void
     */
    public function down()
    {
        Permission::whereIn( 'namespace', [
            'stocktransfers.create.transaction',
            'stocktransfers.read.transaction',
            'stocktransfers.update.transaction',
            'stocktransfers.delete.transaction',
        ])->each( function( $permission ) {
            $permission->removeFromRoles();
            $permission->delete();
        });
    }
}
