<?php
namespace Modules\NsStockTransfers\Providers;

use App\Classes\Hook;
use App\Services\NotificationService;
use App\Services\ProductCategoryService;
use App\Services\ProductService;
use App\Services\TaxService;
use App\Services\UnitService;
use Illuminate\Support\Facades\Event;
use Illuminate\Support\ServiceProvider;
use Modules\NsMultiStore\Events\MultiStoreApiRoutesLoadedEvent;
use Modules\NsMultiStore\Events\MultiStoreWebRoutesLoadedEvent;
use Modules\NsStockTransfers\Events\NsStockTransfersEvent;
use Modules\NsStockTransfers\Events\TransactionAfterApproveEvent;
use Modules\NsStockTransfers\Events\TransactionAfterProcessedEvent;
use Modules\NsStockTransfers\Events\TransactionAfterRejectedEvent;
use Modules\NsStockTransfers\Services\StockTransferService;
use Modules\NsStockTransfers\Settings\GeneralSettings;

class EventServiceProvider extends ServiceProvider
{
    public function register()
    {
        Hook::addFilter( 'ns-dashboard-menus', [ NsStockTransfersEvent::class, 'dashboardMenus' ], 30 );
        Hook::addFilter( 'ns-crud-resource', [ NsStockTransfersEvent::class, 'registerCrud' ]);

        Event::listen( function( MultiStoreWebRoutesLoadedEvent $event ) {
            include( NSTRANSFERPATH . DIRECTORY_SEPARATOR . 'Routes' . DIRECTORY_SEPARATOR . 'multistore.php' );
        });

        Event::listen( function( MultiStoreApiRoutesLoadedEvent $event ) {
            include( NSTRANSFERPATH . DIRECTORY_SEPARATOR . 'Routes' . DIRECTORY_SEPARATOR . 'api.php' );
        });

        Event::listen( TransactionAfterApproveEvent::class, [ NsStockTransfersEvent::class, 'emitNotifications' ]);
        Event::listen( TransactionAfterRejectedEvent::class, [ NsStockTransfersEvent::class, 'emitNotifications' ]);
        Event::listen( TransactionAfterProcessedEvent::class, [ NsStockTransfersEvent::class, 'emitNotifications' ]);

        $this->app->singleton( StockTransferService::class, function() {
            return new StockTransferService(
                app()->make( ProductService::class ),
                app()->make( UnitService::class ),
                app()->make( ProductCategoryService::class ),
                app()->make( TaxService::class ),
                app()->make( NotificationService::class )
            );
        });

        Hook::addFilter( 'ns.settings', function( $class, $identifier ) {
            switch( $identifier ) {
                case 'ns.stocktransfers.settings': return new GeneralSettings; break;
                default: return $class;
            }
        }, 10, 2 );
    }
}