@extends( 'layout.dashboard' )

@section( 'layout.dashboard.body.with-header' )
    @include( 'common.dashboard.title' )
    <stock-invoice inline-template>
        <div>
            <div class="flex -mx-2">
                <div class="px-2 flex">
                    <div class="ns-button hover-info">
                        <button @click="print()" class="rounded flex justify-between shadow py-1 items-center px-2">
                            <i class="las la-print text-xl"></i>
                            <span class="pl-2">{{ __( 'Print' ) }}</span>
                        </button>
                    </div>
                </div>
                <div class="px-2 flex">
                    <div class="ns-button hover-info">
                        <button @click="printLabels()" class="rounded flex justify-between shadow py-1 items-center px-2">
                            <i class="las la-print text-xl"></i>
                            <span class="pl-2">{{ __( 'Print Labels' ) }}</span>
                        </button>
                    </div>
                </div>
                <div class="px-2 flex">
                    <div class="ns-button hover-info">
                        <a href="{{ ns()->route( 'ns.stocktransfer-list' ) }}" class="rounded flex justify-between shadow py-1 items-center px-2">
                            <i class="las la-arrow-left"></i>
                            <span class="pl-2">{{ __( 'Return' ) }}</span>
                        </a>
                    </div>
                </div>
                <div class="px-2 flex">
                    <div class="ns-button hover-info">
                        <button @click="openSettings()" class="cursor-pointer rounded flex justify-between shadow py-1 items-center px-2">
                            <i class="las la-cog"></i>
                            <span class="pl-2">{{ __( 'Settings' ) }}</span>
                        </button>
                    </div>
                </div>
            </div>
            <div id="stock-invoice" class="anim-duration-500 fade-in-entrance">
                <div class="flex w-full">
                    <div class="my-4 flex justify-between w-full">
                        <div class="text-primary flex-auto">
                            <div class="row -mx-2 flex">
                                <div class="w-1/2 px-2">
                                    <ul>
                                        <li class="pb-1 border-b border-dashed">{{ sprintf( __( 'Name : %s' ), $transaction->name ) }}</li>
                                        <li class="pb-1 border-b border-dashed">{{ sprintf( __( 'Date : %s' ), ns()->date->getFormatted( $transaction->created_at ) ) }}</li>
                                        <li class="pb-1 border-b border-dashed">{{ __( 'Document : Stock Transfer Invoice' ) }}</li>
                                        <li class="pb-1 border-b border-dashed">{{ sprintf( __( 'Status : %s' ), $transaction->human_status ) }}</li>
                                    </ul>
                                </div>
                                <div class="w-1/2 px-2">
                                    <ul>
                                        <li class="pb-1 border-b border-dashed">{{ sprintf( __( 'Source Store : %s' ), $transaction->sourceStore->name ) }}</li>
                                        <li class="pb-1 border-b border-dashed">{{ sprintf( __( 'Destination Store : %s' ), $transaction->destinationStore->name ) }}</li>
                                        <li class="pb-1 border-b border-dashed">{{ sprintf( __( 'Initialized By : %s' ), $transaction->user->username ) }}</li>
                                    </ul>
                                </div>
                            </div>
                        </div>
                        <div                                                                                                                                                                                                                                >
                            <img src="{{ ns()->option->get( 'ns_store_rectangle_logo', 'foo' ) }}" alt="{{ ns()->option->get( 'ns_store_name' ) }}">
                        </div>
                    </div>
                </div>
                <div class="table w-full ns-table">
                    <table class="table w-full shadow-sm">
                        <thead>
                            <tr>
                                <th class="border p-3 text-left">{{ __( 'Product' ) }}</th>
                                <th class="border p-3 text-center">{{ __( 'Unit' ) }}</th>
                                <th class="border p-3 text-center">{{ __( 'Unit Price' ) }}</th>
                                <th class="border p-3 text-center">{{ __( 'Quantity' ) }}</th>
                                <th class="border p-3 text-center">{{ __( 'Total Price' ) }}</th>
                            </tr>
                        </thead>
                        <tbody>
                            @foreach( $transaction->products as $product )
                            <tr>
                                <td class="p-2 border text-left">{{ $product->name }} - ({{ $product->barcode }})</td>
                                <td class="p-2 border text-center">{{ $product->unit_name }}</td>
                                <td class="p-2 border text-center">{{ ns()->currency->define( $product->unit_price ) }}</td>
                                <td class="p-2 border text-center">{{ $product->quantity }}</td>
                                <td class="p-2 border text-center">{{ ns()->currency->define( $product->unit_price )->multiplyBy( $product->quantity ) }}</td>
                            </tr>
                            @endforeach
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </stock-invoice>
@endsection

@section( 'layout.dashboard.footer.inject' )
    @parent
<script>
    const nsStockSettings   =   Vue.component( 'stock-settings', {
        template: `
        <div class="shadow-xl w-95vw md:w-4/5-screen ns-box">
            <div class="flex justify-between items-center p-2 ns-box-header border-b">
                <h3>{{ __m( 'Settings', 'NsStockTransfers' ) }}</h3>
                <div>
                    <ns-close-button @click="closePopup()"></ns-close-button>
                </div>
            </div>
            <div class="flex ns-box-body">
                <div class="w-full md:w-1/2 bg-gray-800">
                    <div class="p-8">
                        <div :style="{ paddingTop: (config.vertical_padding|0) + 'px', paddingBottom: (config.vertical_padding|0) + 'px', paddingLeft: (config.horizontal_padding|0) + 'px', paddingRight: (config.horizontal_padding|0) + 'px' }" class="m-6 bg-white text-black flex flex-col justify-center items-center">
                            <h3 class="font-bold text-2xl text-black">{{ __m( 'Product Name', 'NsStockTransfers' ) }}</h3>
                            <p class="text-black">{{ __m( 'Quantity :', 'NsStockTransfers' ) }} 10, {{ __m( 'Barcode :', 'NsStockTransfers' ) }} sample-barcode</p>
                            <div :style="{ height: (config.barcode_height|0) + 'px' }" class="bg-black w-1/2"></div>
                        </div>
                    </div>
                </div>
                <div class="w-full md:w-1/2">
                    <div class="p-2">
                        <ns-field :field="field" v-for="field of fields"/>
                    </div>
                </div>
            </div>
            <div class="p-2 flex justify-between border-t ns-box-footer">
                <ns-button @click="saveSettings()" type="info">{{ __m( 'Save', 'NsStockTransfers' ) }}</ns-button>
            </div>
        </div>
        `,
        data() {
            return {
                fields: [],
                savedUrl: null,
                validation: new FormValidation,
            }
        },
        mounted() {
            this.popupCloser();

            this.fields     =   this.validation.createFields( this.$popupParams.fields );
        },
        computed: {
            config() {                
                return this.validation.extractFields( this.fields );
            },
        },
        watch: {
            config() {
                if ( this.config.nps_url && this.config.nps_url.length > 0 && this.savedUrl !== this.config.nps_url ) {
                    this.savedUrl   =   this.config.nps_url;
                    nsHttpClient.get( `${this.config.nps_url}/api/printers` )
                        .subscribe({
                            next: printers => {
                                this.fields.forEach( field => {
                                    if ( field.name === 'printer' ) {
                                        field.options   =   printers.map( printer => {
                                            return {
                                                label: printer.name,
                                                value: printer.name
                                            }
                                        })
                                    }
                                })
                            }
                        })
                }
            }
        },
        methods: {
            popupCloser,
            popupResolver,
            saveSettings() {
                this.popupResolver( this.config );
            },
            closePopup() {
                this.popupResolver( false );
            },
        }
    });
    
    Vue.component( 'stock-invoice', {
        data() {
            return {
                config: false,
                products: <?php echo json_encode( $transaction->products );?>,
                settingsFields: [
                    {
                        label: __m( 'Vertical Padding' ),
                        name: 'vertical_padding',
                        type: 'text'
                    }, {
                        label: __m( 'Horizontal Padding' ),
                        name: 'horizontal_padding',
                        type: 'text'
                    }, {
                        label: __m( 'Barcode Height' ),
                        name: 'barcode_height',
                        type: 'text'
                    },  {
                        label: __m( 'Paper Size (mm)' ),
                        name: 'paper_size',
                        type: 'text',
                        value: 80
                    }, {
                        label: __m( 'NPS URL' ),
                        name: 'nps_url',
                        type: 'text',
                    }, {
                        label: __m( 'Printer' ),
                        name: 'printer',
                        options: [],
                        type: 'select',
                    }
                ]
            }
        },
        methods: {
            print() {
                this.$htmlToPaper( 'stock-invoice' );
            },
            printLabels() {
                this.products.forEach( product => {
                    nsHttpClient.post( `${this.config.nps_url}/api/print`, {
                        printer: this.config.printer,
                        content: `
                        <${'?'}xml version="1.0" encoding="UTF-8"?>
                        <document>
                            <align mode=\"center\">
                                <h4>
                                    <text-line>${product.name}</text-line>
                                </h4>
                                <text-line>{{ __m( 'Quantity', 'NsStockTransfers' ) }} : ${product.quantity}</text-line>
                                <barcode type=\"BARCODE_CODE39\">${product.barcode}</barcode>
                            </align>
                            <paper-cut></paper-cut>
                        </document>
                        `
                    }).subscribe({
                        next: result => {
                            console.log( result );
                        },
                        error: error => {
                            console.log( error );
                        }
                    })
                })
            },
            async openSettings() {
                try {
                    const config   =   await new Promise( ( resolve, reject ) => {
                        Popup.show( nsStockSettings, { resolve, reject, fields: this.settingsFields, config : this.config });
                    });

                    if ( config !== false ) {
                        this.config     =   config;
                    }

                } catch( exception ) {
                    // ...
                }
            }
        }
    })
</script>
@endsection