@extends( 'layout.dashboard' )
@section( 'layout.dashboard.body.with-header' )
@include( 'common.dashboard.title' )
<transfer-request 
    v-cloak
    src="{{ ns()->url( '/api/nexopos/v4/crud/ns.stocktransfers-requests/form-config' . ( isset( $transaction ) ? '/' . $transaction->id : '' )  ) }}"
    fetch-url="{{ ns()->url( '/api/nexopos/v4/stock-transfers/search' . ( isset( $transaction ) ? '/' . $transaction->id : '' ) ) }}"
    submit-method="{{ isset( $transaction ) ? 'PUT' : 'POST' }}"
    submit-url="{{ ns()->url( '/api/nexopos/v4/stock-transfers/submit' . ( isset( $transaction ) ? '/' . $transaction->id : '' ) ) }}"
    return-url="{{ ns()->route( 'ns.stocktransfer-list' ) }}"
    inline-template>
    <div>
    @include( 'NsStockTransfers::transfers.template' )
    </div>
</transfer-request>
@endsection

@section( 'layout.dashboard.footer.inject' )
    @parent
<script>
nsExtraComponents.createTransferRequest     =   Vue.component( 'transfer-request', {
    mounted() {
        this.loadForm();
    },
    props: [ 'src', 'create-url', 'field-class', 'return-url', 'fetch-url', 'submit-url', 'submit-method', 'disable-tabs' ],
    computed: {
        activeTabFields() {
            for( let identifier in this.form.tabs ) {
                if ( this.form.tabs[ identifier ].active ) {
                    return this.form.tabs[ identifier ].fields;
                }
            }
            return [];
        },
        activeTabIdentifier() {
            for( let identifier in this.form.tabs ) {
                if ( this.form.tabs[ identifier ].active ) {
                    return identifier;
                }
            }
            return false;
        }
    },
    watch: {
        searchExpression() {
            if ( this.searchExpression.length > 0 ) {
                clearTimeout( this.searchTimeout );
                this.searchTimeout  =   setTimeout( () => {
                    this.searchProduct();
                }, 800 );
            }
        }
    },
    data() {
        return {
            products: [],
            searchTimeout: null,
            searchExpression: '',
            searchResults: [],
            form: {},
            globallyChecked: false,
            formValidation: new FormValidation,
            rows: []
        }
    },
    methods: {
        formLoaded( form ) {
            this.form   =   form;
        },
        addProduct( product ) {
            product.unit_quantity   =   {}
            product.quantity        =   0;
            product.product_id      =   product.id;
            delete product.id;

            this.form.products.unshift( product );
            
            this.searchResults  =   [];

            return nsSnackBar.success( `Product has been added.` ).subscribe();
        },
        searchProduct() {
            nsHttpClient.post( this.fetchUrl, { search : this.searchExpression, ...this.form.tabs.general } )
                .subscribe( result => {
                    this.searchExpression   =   '';
                    if ( result.length > 1 ) {
                        this.searchResults  =   result;
                    } else if ( result.length === 1 ) {
                        this.addProduct( result[0] );
                    } else {
                        nsSnackBar.error( 'No product has been found.' ).subscribe();
                    }
                }, ( error ) => {
                    nsSnackBar.error( error.message ).subscribe();
                })
        },
        toggle( identifier ) {
            for( let key in this.form.tabs ) {
                this.form.tabs[ key ].active    =   false;
            }
            this.form.tabs[ identifier ].active     =   true;
        },
        /**
         * Dpeca
         * @param {Object} e Something
         * @deprecated
         */
        handleShowOptions( e ) {
            this.rows.forEach( row => {
                if ( row.$id !== e.$id ) {
                    row.$toggled    =   false;
                }
            });
        },
        submit() {
            if ( this.formValidation.validateForm( this.form ).length > 0 ) {
                return nsSnackBar.error( this.$slots[ 'error-invalid-form' ] ? this.$slots[ 'error-invalid-form' ][0].text : 'No error message provided for having an invalid form.', this.$slots[ 'okay' ] ? this.$slots[ 'okay' ][0].text : 'OK' )
                    .subscribe();
            }

            if ( this.submitUrl === undefined ) {
                return nsSnackBar.error( this.$slots[ 'error-no-submit-url' ] ? this.$slots[ 'error-no-submit-url' ][0].text : 'No error message provided for not having a valid submit url.', this.$slots[ 'okay' ] ? this.$slots[ 'okay' ][0].text : 'OK' )
                    .subscribe();
            }

            if ( this.form.products.length === 0 ) {
                return nsSnackBar.error( this.$slots[ 'error-no-product-provided' ] ? this.$slots[ 'error-no-product-provided' ][0].text : 'Consider providing at least one product before proceeding.', this.$slots[ 'okay' ] ? this.$slots[ 'okay' ][0].text : 'OK' )
                    .subscribe();
            }

            if ( this.form.products.filter( product => product.quantity === 0 ).length > 0 ) {
                return nsSnackBar.error( this.$slots[ 'error-quantity-provided' ] ? this.$slots[ 'error-quantity-provided' ][0].text : 'One or more products has "0" as a quantity provided.', this.$slots[ 'okay' ] ? this.$slots[ 'okay' ][0].text : 'OK' )
                    .subscribe();
            }

            this.formValidation.disableForm( this.form );

            const data  =   {
                products:   this.form.products,
                ...this.formValidation.extractForm( this.form )
            };

            nsHttpClient[ this.submitMethod ? this.submitMethod.toLowerCase() : 'post' ]( this.submitUrl, data )
                .subscribe( result => {
                    if ( result.status === 'success' ) {
                        if ( this.returnUrl && this.returnUrl.length > 0 ) {
                            return document.location   =   this.returnUrl;
                        }
                        this.$emit( 'save', result );
                    }
                    this.formValidation.enableForm( this.form );
                }, ( error ) => {
                    nsSnackBar.error( error.message, undefined, {
                        duration: 5000
                    }).subscribe();
                    
                    this.formValidation.triggerError( this.form, error );
                    this.formValidation.enableForm( this.form );
                })
        },
        handleGlobalChange( event ) {
            this.globallyChecked    =   event;
            this.rows.forEach( r => r.$checked = event );
        },
        loadForm() {
            const request   =   nsHttpClient.get( `${this.src}` );

            request.subscribe( (f) => {
                this.form    =   this.parseForm( f.form );
                this.form.products.forEach( product => {
                    const selectedUnitQuantity      =   product.unit_quantities
                        .filter( _unit_quantity => _unit_quantity.id === product.unit_quantity.id );
                    product.unit_quantity           =   selectedUnitQuantity.length > 0 ? selectedUnitQuantity[0] : {};
                });
            });
        },
        removeProduct( product ) {
            if ( product.id !== undefined ) {
                return Popup.show( nsConfirmPopup, {
                    title: 'Confirm Your Action',
                    message: 'This product has a reference on the database. Erasing this product will remove the record from the current stock transfer. Would you proceed ?',
                    onAction:  ( action ) => {
                        if ( action ) {
                            return this.deleteProduct( product );
                        }
                    }
                });
            } else {
                return Popup.show( nsConfirmPopup, {
                    title: 'Confirm Your Action',
                    message: 'Would you like to remove this product ?',
                    onAction:  ( action ) => {
                        if ( action ) {
                            return this.removeProductFromIndex( product );
                        }
                    }
                });
            }            
        },
        removeProductFromIndex( product ) {
            this.form.products.splice( this.form.products.indexOf( product ), 1 );
        },
        deleteProduct( product ) {
            nsHttpClient.delete( '/api/nexopos/v4/stock-transfers/products/' + product.id )
                .subscribe( result => {
                    nsSnackBar.success( result.message ).subscribe();
                    this.removeProductFromIndex( product );
                    this.$forceUpdate();
                }, ( error ) => {
                    nsSnackBar.error( error.message ).subscribe();
                });
        },
        setQuantity( product ) {
            if ( product.unit_quantity === undefined || Object.values( product.unit_quantity ).length === 0 ) {
                return nsSnackBar.error( 'Please select a unit before defining the quantity.' ).subscribe();
            }

            const promise   =   new Promise( ( resolve, reject ) => {
                Popup.show( nsProcurementQuantity, { quantity : product.quantity, resolve, reject })
            });

            promise.then( data => {
                const index             =   this.form.products.indexOf( product );
                const usedQuantities    =   this.form.products.filter( ( _product, _index ) => {
                    /**
                     * select every product that has the
                     * same identifier and the same id
                     * but not the same index as we don't want to consider
                     * the stock provided on that product
                     */
                    if ( Object.values( _product.unit_quantity ).length > 0 && _index !== index ) {
                        return _product.id === product.id && _product.unit_quantity.unit.id === product.unit_quantity.unit.id
                    }
                }).map( p => p.quantity );

                let quantityUsed    =   0;
                if( usedQuantities.length > 0 ) {
                    quantityUsed    =   usedQuantities.reduce( ( b, a ) => b + a );
                }

                /**
                 * if the quantity defined + the quantity used
                 * is greater than the quantity available, we should 
                 * forbid the transaction.
                 */
                const remaining     =   product.unit_quantity.quantity - ( quantityUsed );

                if ( ( quantityUsed + data.quantity ) > product.unit_quantity.quantity ) {
                    return nsSnackBar
                        .error( 'Unable to define that quantity as there is no more stock remaining (%s)'.replace( '%s', remaining ) )
                        .subscribe();
                }

                this.form.products[ index ].quantity     =   data.quantity;
                this.$forceUpdate();
            }).catch( e => e );
        },
        parseForm( form ) {
            form.main.value     =   form.main.value === undefined ? '' : form.main.value;
            form.main           =   this.formValidation.createFields([ form.main ])[0];
            let index           =   0;

            for( let key in form.tabs ) {
                if ( index === 0 ) {
                    form.tabs[ key ].active  =   true;
                }

                form.tabs[ key ].active     =   form.tabs[ key ].active === undefined ? false : form.tabs[ key ].active;
                form.tabs[ key ].fields     =   this.formValidation.createFields( form.tabs[ key ].fields );

                index++;
            }

            return form;
        }
    }
})
</script>
@endsection