@extends( 'layout.dashboard' )

@section( 'layout.dashboard.body.with-header' )
    @include( 'common.dashboard.title' )
    <ns-stock-review inline-template>
        <div>
            <div class="rounded border-2 input-group flex">
                <input placeholder="{{ __m( 'Use The Product barcode...', 'NsStockTransfers' ) }}" v-model="productBarcode" type="text" class="p-2 outline-none flex-auto">
                <button class="px-3 py-2 w-24">{{ __m( 'Search', 'NsStockTranfers' ) }}</button>
            </div>
            <div class="ns-box shadow rounded my-2">
                <div class="md:-mx-4 flex flex-col md:flex-row">
                    <div class="md:px-4 w-full md:w-1/3">
                        <div class="p-4">
                            <h3 class="font-semibold">{{ __m( 'Total Single Items', 'NsStockTransfers' ) }}</h3>
                            <h2 class="text-3xl font-bold">{{ $transaction->products()->count() }}</h2>
                        </div>
                    </div>
                    <div class="md:px-4 w-full md:w-1/3">                    
                        <div class="p-4">
                            <h3 class="font-semibold">{{ __m( 'Total Units', 'NsStockTransfers' ) }}</h3>
                            <h2 class="text-3xl font-bold">{{ $transaction->products()->get()->map( fn( $product ) => $product->quantity )->sum() }}</h2>
                        </div>
                    </div>
                    <div class="md:px-4 w-full md:w-1/3">                    
                        <div class="p-4">
                            <h3 class="font-semibold">{{ __m( 'Missing Products', 'NsStockTransfers' ) }}</h3>
                            <h2 class="text-3xl font-bold">@{{ products.length - counted.length }}</h2>
                        </div>
                    </div>
                </div>
            </div>
            <div class="my-2 ns-box-body shadow">
                <div class="p-2">
                    <table class="w-full ns-table">
                        <thead>
                            <tr>
                                <td class="border font-bold px-3 py-2">{{ __m( 'Product Name', 'NsStockTransfers' ) }}</td>
                                <td class="border font-bold px-3 py-2">{{ __m( 'Quantity', 'NsStockTransfers' ) }}</td>
                            </tr>
                        </thead>
                        <tbody>
                            <tr v-for="product of counted">
                                <td class="px-3 py-2 border">
                                    <h3 class="font-semibold">@{{ product.name }}</h3>
                                    <ul class="flex -mx-1 text-xs">
                                        <li class="px-1">
                                            <span>{{ __m( 'Barcode', 'NsStockTransfers' ) }} :</span>
                                            <span>@{{ product.barcode }}</span>
                                        </li>
                                        <li class="px-1">
                                            <a class="text-error-secondary hover:underline cursor-pointer" @click="removeProduct( product )">{{ __m( 'Remove', 'NsStockTransfers' ) }}</a>
                                        </li>
                                    </ul>
                                </td>
                                <td class="px-3 w-96 py-2 border">
                                    <div class="text-sm flex flex:col md:flex-row">
                                        <div class="md:w-1/3 flex items-center">
                                            <span>
                                            {{ __( 'Expected : ' ) }}
                                            </span>
                                            <span class="p-1 border-b-2 border-dashed cursor-pointer">@{{ product.quantity }}</span>
                                        </div>
                                        <div class="md:w-1/3 flex items-center">
                                            <span>
                                            {{ __( 'Counted : ' ) }}
                                            </span>
                                            <span @click="defineCounted( product )" class="p-1 border-b-2 border-dashed cursor-pointer">@{{ product.available_quantity }}</span>
                                        </div>
                                        <div class="md:w-1/3 flex items-center">
                                            <span>
                                            {{ __( 'Missing : ' ) }}
                                            </span>
                                            <span class="p-1 border-b-2 border-dashed cursor-pointer">@{{ product.missing_quantity }}</span>
                                        </div>
                                    </div>
                                </td>
                            </tr>
                            <tr v-if="counted.length === 0">
                                <td colspan="2" class="px-3 w-96 py-2 border">
                                    <span>{{ __m( 'No products has been counted so far...', 'NsStockTransfers' ) }}</span>
                                </td>
                            </tr>
                        </tbody>
                    </table>
                </div>
                <div class="p-2 flex justify-between border-t ns-box-footer">
                    <div></div>
                    <div>
                        <ns-button v-if="counted.length > 0" @click="proceedStockApproval()" type="info">{{ __m( 'Approve Transfer' ) }}</ns-button>
                        <ns-button v-if="counted.length === 0" @click="proceedStockApproval()" type="danger">{{ __m( 'Reject Transfer' ) }}</ns-button>
                    </div>
                </div>
            </div>
        </div>
    </ns-stock-review>
@endsection

@section( 'layout.dashboard.footer.inject' )
    @parent
    <script>
        const data  =   {
            products: <?php echo json_encode( $transaction->products );?>,
            transfer_id: <?php echo $transaction->id;?>,
            listURL: `{{ ns()->route( 'ns.stocktransfer-list' ) }}`
        }
    </script>
    <script>
        document.addEventListener( 'DOMContentLoaded', () => {
            window.nsReviewQuantity  =   Vue.component( 'ns-review-quantity', {
                components: {
                    nsNumpad: nsComponents.nsNumpad
                },
                mounted() {
                    this.popupCloser();
                    this.input  =   this.$popupParams.quantity;
                },
                data() {
                    return {
                        input: 0,
                    }
                },
                methods: {
                    popupCloser,
                    popupResolver,
                    updateInput( value ) {
                        this.input  =   value;
                    },
                    setValue( value ) {
                        this.popupResolver( value );
                    }
                },
                template: `
                <div class="ns-box shadow-lg w-95vw md:w-2/5-screen">
                    <div class="p-2 flex justify-between items-center border-b ns-box-header">
                        <h3>${ __m( 'Counted Quantities', 'NsStockTranfers' ) }</h3>
                        <div>
                            <ns-close-button @click="popupResolver( false )"></ns-close-button>
                        </div>
                    </div>
                    <div class="p-2 ns-box-body">
                        <div class="mb-2 h-16 elevation-surface border flex items-center justify-center text-2xl">@{{ input }}</div>
                        <ns-numpad :value="input" @next="setValue( $event )" :currency="false" @changed="updateInput( $event )"></ns-numpad>
                    </div>
                </div>
                `
            })
        })
    </script>
    <script>
        Vue.component( 'ns-stock-review', {
            watch: {
                productBarcode() {
                    clearTimeout( this.debouncer );
                    this.debouncer  =   setTimeout( () => {
                        if ( this.productBarcode.length > 0 ) {
                            this.addProduct( this.productBarcode );
                        }
                    }, 1000 );
                }
            },
            data : () => {
                return {
                    products: data.products,
                    debouncer: null,
                    productBarcode: "",
                    counted: [],
                }
            },
            mounted() {
                // ...
            },
            methods: {
                __m,
                removeProduct( product ) {
                    const index     =   this.counted.indexOf( product );
                    this.counted.splice( index, 1 );
                },
                completeStockTransfer() {
                    nsHttpClient.post( `/api/nexopos/v4/stock-transfers/confirm-transfer/${data.transfer_id}`, {
                        counted: this.counted
                    }).subscribe({
                        next: ( result ) => {
                            this.counted    =   [];
                            nsSnackBar.success( result.message, __m( 'Ok', 'NsStockTransfers' ) ).subscribe();
                            document.location   =   data.listURL;
                        },
                        error: ( error ) => {
                            nsSnackBar.error( error.message, __m( 'Ok', 'NsStockTransfers' ) ).subscribe();
                        }
                    })
                },
                rejectStockTransfer() {
                    nsHttpClient.get( `/api/nexopos/v4/stock-transfers/reject/${data.transfer_id}`).subscribe({
                        next: ( result ) => {
                            this.counted    =   [];
                            nsSnackBar.success( result.message, __m( 'Ok', 'NsStockTransfers' ) ).subscribe();
                            document.location   =   data.listURL;
                        },
                        error: ( error ) => {
                            nsSnackBar.error( error.message, __m( 'Ok', 'NsStockTransfers' ) ).subscribe();
                        }
                    })
                },
                proceedStockApproval() {
                    if ( this.counted.length === 0 ) {
                        return Popup.show( nsConfirmPopup, {
                            title: __m( 'Warning Before Proceeding', 'NsStockTransfers' ),
                            message: __m( `You\'ve not added any product. If you continue, the stock transfer will be rejected and the inventory returned to the source.`, 'NsStockTransfers' ),
                            onAction: ( action ) => {
                                if ( action ) {
                                    if ( this.counted.length === 0 ) {
                                        this.rejectStockTransfer();
                                    } else {
                                        this.completeStockTransfer();
                                    }
                                }
                            }
                        })
                    }

                    return Popup.show( nsConfirmPopup, {
                        title: __m( 'Confirm Your Action', 'NsStockTransfers' ),
                        message: __m( `You're about to approve a stock transfer. Missing products will be returned to the source. Would you like to proceed ?`, 'NsStockTransfers' ),
                        onAction: ( action  ) => {
                            if ( action ) {
                                this.completeStockTransfer();
                            }
                        }
                    })
                },
                async defineCounted( product ) {
                    try {
                        const quantity   =   await new Promise( ( resolve, reject ) => {
                            Popup.show( nsReviewQuantity, { quantity : product.available_quantity, resolve, reject })
                        });

                        product.available_quantity      =   quantity;
                        product.missing_quantity        =   product.quantity - product.available_quantity;
                    } catch( exception ) {
                        // ...
                    }
                },
                
                addProduct( barcodeProduct ) {
                    const alreadyAdded  =   this.counted
                        .filter( product => product.barcode === barcodeProduct )
                        .length > 0;

                    if ( alreadyAdded ) {
                        this.productBarcode     =   '';
                        return nsSnackBar.error( __m( 'The product is already added.', 'NsStockTransfers' ) ).subscribe();
                    }

                    const product   =   this.products.filter( product => product.barcode === barcodeProduct );

                    if ( product.length === 0 ) {
                        this.productBarcode     =   '';
                        return nsSnackBar.error( __m( 'Invalid barcode provided.', 'NsStockTransfers' ) ).subscribe();
                    }

                    this.counted.push({
                        ...product[0],
                        ...{
                            available_quantity: 0,
                            missing_quantity: product[0].quantity
                        }
                    });

                    this.productBarcode     =   '';
                    return nsSnackBar.success( __m( 'The product has been added.', 'NsStockTransfers' ) ).subscribe();
                }
            }
        });
    </script>
@endsection