<div class="form flex-auto" id="crud-form">
    <div v-if="Object.values( form ).length === 0" class="flex items-center justify-center h-full">
        <ns-spinner />
    </div>
    <div v-if="Object.values( form ).length > 0">
        <div class="flex flex-col">
            <div class="flex justify-between items-center" v-if="form.main">
                <label for="title" class="font-bold my-2 text-primary">@{{ form.main.label }}</label>
                <div for="title" class="text-sm my-2 text-primary">
                    <a v-if="returnUrl" :href="returnUrl" class="rounded-full border ns-inset-button primary px-2 py-1">{{ __m( 'Go Back', 'NsStockTransfers' ) }}</a>
                </div>
            </div>
            <div :class="form.main.disabled ? 'disabled' : form.main.errors.length > 0 ? 'error' : 'info'" class="flex border-2 rounded input-group overflow-hidden">
                <input v-model="form.main.value" 
                    @blur="formValidation.checkField( form.main )" 
                    @change="formValidation.checkField( form.main )" 
                    :disabled="form.main.disabled"
                    type="text" 
                    :class="form.main.disabled ? 'disabled' : ''"
                    class="flex-auto text-primary outline-none h-10 px-2">
                <button :disabled="form.main.disabled" :class="form.main.disabled ? 'disabled' : form.main.errors.length > 0 ? 'error' : 'info'" @click="submit()" class="outline-none px-4 h-10 border-l"><slot name="save">{{ __m( 'Save', 'NsStockTransfers' ) }}</slot></button>
            </div>
            <p class="text-xs text-secondary py-1" v-if="form.main.description && form.main.errors.length === 0">@{{ form.main.description }}</p>
            <p class="text-xs py-1 error" v-for="error of form.main.errors">
                <span v-if="error.identifier=== 'required'"><slot name="error-required">@{{ error.identifier }}</slot></span>
                <span v-if="error.identifier=== 'invalid'"><slot name="error-invalid">@{{ error.message }}</slot></span>
            </p>
        </div>
        <div id="tabs-container" class="my-5 ns-tab" v-if="activeTabIdentifier === 'general'">
            <div class="header flex" style="margin-bottom: -1px;">
                <div v-for="( tab , identifier ) of form.tabs" 
                    @click="toggle( identifier )" 
                    :class="tab.active ? 'active border border-b-transparent' : 'border inactive'" 
                    class="tab rounded-tl rounded-tr border tab  px-3 py-2 text-primary cursor-pointer" style="margin-right: -1px">@{{ tab.label }}</div>
            </div>
            <div class="border ns-tab-item p-4">
                <div class="-mx-4 flex flex-wrap">
                    <div :class="fieldClass || 'px-4 w-full md:w-1/2 lg:w-1/3'" v-for="field of activeTabFields">
                        <ns-field @blur="formValidation.checkField( field )" @change="formValidation.checkField( field )" :field="field"/>
                    </div>
                </div>
            </div>
        </div>
        <div id="tabs-container" class="my-5 ns-tab" v-if="activeTabIdentifier === 'table'">
            <div class="header flex" style="margin-bottom: -1px;">
                <div v-for="( tab , identifier ) of form.tabs" 
                    @click="toggle( identifier )" 
                    :class="tab.active ? 'active border border-b-transparent' : 'border inactive'" 
                    class="tab rounded-tl rounded-tr border tab  px-3 py-2 text-primary cursor-pointer" style="margin-right: -1px">@{{ tab.label }}</div>
            </div>
            <div class="border p-4 ns-tab-item">
                <div class="header mb-2 border-b">
                    <div class="flex rounded overflow-hidden border-2 input-group info">
                        <input ref="search" v-model="searchExpression" type="text" class="p-2 flex-auto">
                        <button class="px-3 py-2">{{ __( 'Search' ) }}</button>
                    </div>
                    <div class="h-0 relative w-100 anim-duration-100 zoom-in-entrance" v-if="searchResults.length > 0">
                        <div class="shadow absolute w-full">
                            <ul class="ns-vertical-menu">
                                <li @click="addProduct( result )" v-for="result of searchResults" class="cursor-pointer p-2 border-b text-primary">@{{ result.name }}</li>
                            </ul>
                        </div>
                    </div>
                </div>
                <div class="search-table">
                    <table class="w-full ns-table">
                        <thead class="text-primary ">
                            <tr>
                                <td class="border text-left px-2 py-1">{{ __m( 'Product Name', 'NsStockTransfers' ) }}</td>
                                <td width="150" class="border text-right px-2 py-1">{{ __m( 'Unit', 'NsStockTransfers' ) }}</td>
                                <td width="150" class="border text-right px-2 py-1">{{ __m( 'Quantity', 'NsStockTransfers' ) }}</td>
                            </tr>
                        </thead>
                        <tbody class="text-secondary">
                            <tr v-for="product of form.products">
                                <td class="border p-2">
                                    <div class="flex flex-col">
                                        <span class="text-lg">@{{ product.name }}</span>
                                        <div class="flex -mx-2">
                                            <div class="px-2">
                                                <span @click="removeProduct( product )" class="text-sm text-error-secondary hover:underline cursor-pointer">Delete</span>
                                            </div>
                                            <div class="px-2">
                                                <small class="text-s hover:underline cursor-pointer">{{ __m( 'SKU', 'NsStockTransfers' ) }} : @{{ product.sku }}</small>
                                            </div>
                                            <div class="px-2">
                                                <small class="text-s hover:underline cursor-pointer">{{ __m( 'Barcode', 'NsStockTransfers' ) }} : @{{ product.barcode }}</small>
                                            </div>
                                        </div>
                                    </div>
                                </td>
                                <td class="border p-2" class="border-2">
                                    <div class="ns-select">
                                        <select v-model="product.unit_quantity" class="w-full p-2">
                                            <option :value="option" v-for="option of product.unit_quantities">@{{ option.unit.name }} (@{{ option.quantity }})</option>
                                        </select>
                                    </div>
                                </td>
                                <td class="border p-2">
                                    <div class="flex items-center justify-center">
                                        <span @click="setQuantity( product )" class="cursor-pointer border-b border-dashed border-info-secondary p-2">@{{ product.quantity }}</span>
                                    </div>
                                </td>
                            </tr>
                            <tr v-if="form.products.length === 0">
                                <td colspan="4" class="text-center p-2">{{ __m( 'Search And Add Some Products...', 'NsStockTransfers' ) }}</td>
                            </tr>
                        </tbody>
                    </table>
                </div>
            </div>
        </div>
    </div>
</div>